@@ -15,21 +15,16 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-"""All :mod:`nfoview` error classes."""
-
-__all__ = ("AffirmationError",)
-
 
 class AffirmationError(Exception):
 
     """
     Something expected to be ``True`` was ``False``.
 
-    :exc:`AffirmationError` is by nature similar to the built-in
-    :exc:`AssertionError`, but without the special reliance on
-    :const:`__debug__` and given optimization options. :exc:`AffirmationError`
-    can be used to provide essential checks of boolean values instead of
-    optional debug checks.
+    :exc:`AffirmationError` is like :exc:`AssertionError`, but without
+    the special reliance on :const:`__debug__` and given optimization options.
+    :exc:`AffirmationError` can be used to provide essential checks of boolean
+    values instead of optional debug checks.
     """
 
     pass
